import { onSnapshot, addDoc, setDoc, deleteDoc, runTransaction, namedQuery as namedQuery$1, getDoc, getDocFromServer, getDocFromCache, getDocs, getDocsFromServer, getDocsFromCache } from 'firebase/firestore';
import { useRef, useEffect } from 'react';
import { useQueryClient, hashQueryKey, useQuery, useInfiniteQuery, useMutation } from 'react-query';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var Completer = /** @class */ (function () {
    function Completer() {
        var _this = this;
        this.resolved = false;
        this.complete = function () { return null; };
        this.reject = function () { return null; };
        this.promise = new Promise(function (resolve, reject) {
            _this.complete = function (value) {
                _this.resolved = true;
                resolve(value);
            };
            _this.reject = reject;
        });
    }
    Object.defineProperty(Completer.prototype, "completed", {
        get: function () {
            return this.resolved;
        },
        enumerable: false,
        configurable: true
    });
    return Completer;
}());

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
var counts$2 = {};
var subscriptions$2 = {};
function useFirestoreDocument(key, ref, options, useQueryOptions) {
    var _a;
    var client = useQueryClient();
    var completer = useRef(new Completer());
    var hashFn = (useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.queryKeyHashFn) || hashQueryKey;
    var hash = hashFn(key);
    var isSubscription = !!(options === null || options === void 0 ? void 0 : options.subscribe);
    useEffect(function () {
        if (!isSubscription) {
            getSnapshot(ref, options === null || options === void 0 ? void 0 : options.source)
                .then(function (snapshot) {
                completer.current.complete(snapshot);
            })
                .catch(function (error) {
                completer.current.reject(error);
            });
        }
    }, [isSubscription, hash, completer]);
    useEffect(function () {
        var _a;
        if (isSubscription) {
            (_a = counts$2[hash]) !== null && _a !== void 0 ? _a : (counts$2[hash] = 0);
            counts$2[hash]++;
            // If there is only one instance of this query key, subscribe
            if (counts$2[hash] === 1) {
                subscriptions$2[hash] = onSnapshot(ref, {
                    includeMetadataChanges: options === null || options === void 0 ? void 0 : options.includeMetadataChanges,
                }, function (snapshot) {
                    // Set the data each time state changes.
                    client.setQueryData(key, snapshot);
                    // Resolve the completer with the current data.
                    if (!completer.current.completed) {
                        completer.current.complete(snapshot);
                    }
                }, function (error) { return completer.current.reject(error); });
            }
            else {
                // Since there is already an active subscription, resolve the completer
                // with the cached data.
                completer.current.complete(client.getQueryData(key));
            }
            return function () {
                counts$2[hash]--;
                if (counts$2[hash] === 0) {
                    subscriptions$2[hash]();
                    delete subscriptions$2[hash];
                }
            };
        }
    }, [isSubscription, hash, completer]);
    return useQuery(__assign(__assign({}, useQueryOptions), { queryKey: (_a = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.queryKey) !== null && _a !== void 0 ? _a : key, queryFn: function () { return completer.current.promise; } }));
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
var counts$1 = {};
var subscriptions$1 = {};
function useFirestoreDocumentData(key, ref, options, useQueryOptions) {
    var _a;
    var client = useQueryClient();
    var completer = useRef(new Completer());
    var hashFn = (useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.queryKeyHashFn) || hashQueryKey;
    var hash = hashFn(key);
    var isSubscription = !!(options === null || options === void 0 ? void 0 : options.subscribe);
    useEffect(function () {
        if (!isSubscription) {
            getSnapshot(ref, options === null || options === void 0 ? void 0 : options.source)
                .then(function (snapshot) {
                var _a;
                var data = snapshot.data({
                    serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps,
                });
                if (data && (options === null || options === void 0 ? void 0 : options.idField)) {
                    data = __assign(__assign({}, data), (_a = {}, _a[options.idField] = snapshot.id, _a));
                }
                completer.current.complete(data);
            })
                .catch(function (error) {
                completer.current.reject(error);
            });
        }
    }, [isSubscription, hash, completer]);
    useEffect(function () {
        var _a;
        if (isSubscription) {
            (_a = counts$1[hash]) !== null && _a !== void 0 ? _a : (counts$1[hash] = 0);
            counts$1[hash]++;
            // If there is only one instance of this query key, subscribe
            if (counts$1[hash] === 1) {
                subscriptions$1[hash] = onSnapshot(ref, {
                    includeMetadataChanges: options === null || options === void 0 ? void 0 : options.includeMetadataChanges,
                }, function (snapshot) {
                    var _a;
                    var data = snapshot.data({
                        serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps,
                    });
                    if (data && (options === null || options === void 0 ? void 0 : options.idField)) {
                        data = __assign(__assign({}, data), (_a = {}, _a[options.idField] = snapshot.id, _a));
                    }
                    // Cannot figure out how to get this working without a cast!
                    var _dataWithIdField = data;
                    // Set the data each time state changes.
                    client.setQueryData(key, _dataWithIdField);
                    // Resolve the completer with the current data.
                    if (!completer.current.completed) {
                        completer.current.complete(_dataWithIdField);
                    }
                }, function (error) { return completer.current.reject(error); });
            }
            else {
                // Since there is already an active subscription, resolve the completer
                // with the cached data.
                completer.current.complete(client.getQueryData(key));
            }
            return function () {
                counts$1[hash]--;
                if (counts$1[hash] === 0) {
                    subscriptions$1[hash]();
                    delete subscriptions$1[hash];
                }
            };
        }
    }, [isSubscription, hash, completer]);
    return useQuery(__assign(__assign({}, useQueryOptions), { queryKey: (_a = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.queryKey) !== null && _a !== void 0 ? _a : key, queryFn: function () { return completer.current.promise; } }));
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
function useFirestoreInfiniteQuery(key, initialQuery, getNextQuery, getPreviousQuery, options, useInfiniteQueryOptions) {
    var _a;
    return useInfiniteQuery({
        queryKey: (_a = useInfiniteQueryOptions === null || useInfiniteQueryOptions === void 0 ? void 0 : useInfiniteQueryOptions.queryKey) !== null && _a !== void 0 ? _a : key,
        queryFn: function (ctx) {
            var _a;
            return __awaiter(this, void 0, void 0, function () {
                var query;
                return __generator(this, function (_b) {
                    query = (_a = ctx.pageParam) !== null && _a !== void 0 ? _a : initialQuery;
                    return [2 /*return*/, getQuerySnapshot(query, options === null || options === void 0 ? void 0 : options.source)];
                });
            });
        },
        getNextPageParam: function (snapshot) {
            return getNextQuery(snapshot);
        },
        getPreviousPageParam: function (snapshot) {
            return getPreviousQuery(snapshot);
        },
    });
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
function useFirestoreInfiniteQueryData(key, initialQuery, getNextQuery, getPreviousQuery, options, useInfiniteQueryOptions) {
    var _a;
    return useInfiniteQuery({
        queryKey: (_a = useInfiniteQueryOptions === null || useInfiniteQueryOptions === void 0 ? void 0 : useInfiniteQueryOptions.queryKey) !== null && _a !== void 0 ? _a : key,
        queryFn: function (ctx) {
            var _a;
            return __awaiter(this, void 0, void 0, function () {
                var query, snapshot;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            query = (_a = ctx.pageParam) !== null && _a !== void 0 ? _a : initialQuery;
                            return [4 /*yield*/, getQuerySnapshot(query, options === null || options === void 0 ? void 0 : options.source)];
                        case 1:
                            snapshot = _b.sent();
                            return [2 /*return*/, snapshot.docs.map(function (doc) {
                                    var _a;
                                    var data = doc.data({ serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps });
                                    if (options === null || options === void 0 ? void 0 : options.idField) {
                                        data = __assign(__assign({}, data), (_a = {}, _a[options.idField] = doc.id, _a));
                                    }
                                    return data;
                                })];
                    }
                });
            });
        },
        getNextPageParam: function (data) {
            return getNextQuery(data);
        },
        getPreviousPageParam: function (data) {
            return getPreviousQuery(data);
        },
    });
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
var counts = {};
var subscriptions = {};
function useFirestoreQuery(key, query, options, useQueryOptions) {
    var _a;
    var client = useQueryClient();
    var completer = useRef(new Completer());
    var hashFn = (useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.queryKeyHashFn) || hashQueryKey;
    var hash = hashFn(key);
    var isSubscription = !!(options === null || options === void 0 ? void 0 : options.subscribe);
    useEffect(function () {
        if (!isSubscription) {
            resolveQuery(query)
                .then(function (resolvedQuery) {
                return getQuerySnapshot(resolvedQuery, options === null || options === void 0 ? void 0 : options.source);
            })
                .then(function (snapshot) {
                completer.current.complete(snapshot);
            })
                .catch(function (error) {
                completer.current.reject(error);
            });
        }
    }, [isSubscription, hash, completer]);
    useEffect(function () {
        var _a;
        if (isSubscription) {
            (_a = counts[hash]) !== null && _a !== void 0 ? _a : (counts[hash] = 0);
            counts[hash]++;
            // If there is only one instance of this query key, subscribe
            if (counts[hash] === 1) {
                resolveQuery(query)
                    .then(function (resolveQuery) {
                    subscriptions[hash] = onSnapshot(resolveQuery, {
                        includeMetadataChanges: options === null || options === void 0 ? void 0 : options.includeMetadataChanges,
                    }, function (snapshot) {
                        // Set the data each time state changes.
                        client.setQueryData(key, snapshot);
                        // Resolve the completer with the current data.
                        if (!completer.current.completed) {
                            completer.current.complete(snapshot);
                        }
                    }, function (error) {
                        completer.current.reject(error);
                    });
                })
                    .catch(function (error) {
                    completer.current.reject(error);
                });
            }
            else {
                // Since there is already an active subscription, resolve the completer
                // with the cached data.
                completer.current.complete(client.getQueryData(key));
            }
            return function () {
                counts[hash]--;
                if (counts[hash] === 0) {
                    subscriptions[hash]();
                    delete subscriptions[hash];
                }
            };
        }
    }, [isSubscription, hash, completer]);
    return useQuery(__assign(__assign({}, useQueryOptions), { queryKey: (_a = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.queryKey) !== null && _a !== void 0 ? _a : key, queryFn: function () { return completer.current.promise; } }));
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
function useFirestoreQueryData(key, query, options, useQueryOptions) {
    var _a, _b;
    var client = useQueryClient();
    var unsubscribe = useRef();
    useEffect(function () {
        return function () { var _a; return (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe); };
    }, []);
    return useQuery(__assign(__assign({}, useQueryOptions), { queryKey: (_a = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.queryKey) !== null && _a !== void 0 ? _a : key, staleTime: ((_b = useQueryOptions === null || useQueryOptions === void 0 ? void 0 : useQueryOptions.staleTime) !== null && _b !== void 0 ? _b : options === null || options === void 0 ? void 0 : options.subscribe) ? Infinity : undefined, queryFn: function () {
            var _a;
            return __awaiter(this, void 0, void 0, function () {
                var _query, snapshot, resolved;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            (_a = unsubscribe.current) === null || _a === void 0 ? void 0 : _a.call(unsubscribe);
                            return [4 /*yield*/, resolveQuery(query)];
                        case 1:
                            _query = _b.sent();
                            if (!!(options === null || options === void 0 ? void 0 : options.subscribe)) return [3 /*break*/, 3];
                            return [4 /*yield*/, getQuerySnapshot(_query, options === null || options === void 0 ? void 0 : options.source)];
                        case 2:
                            snapshot = _b.sent();
                            return [2 /*return*/, snapshot.docs.map(function (doc) {
                                    var _a;
                                    var data = doc.data({
                                        serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps,
                                    });
                                    if (options === null || options === void 0 ? void 0 : options.idField) {
                                        data = __assign(__assign({}, data), (_a = {}, _a[options.idField] = doc.id, _a));
                                    }
                                    return data;
                                })];
                        case 3:
                            resolved = false;
                            return [2 /*return*/, new Promise(function (resolve, reject) {
                                    unsubscribe.current = onSnapshot(_query, {
                                        includeMetadataChanges: options === null || options === void 0 ? void 0 : options.includeMetadataChanges,
                                    }, function (snapshot) {
                                        var docs = snapshot.docs.map(function (doc) {
                                            var _a;
                                            var data = doc.data({
                                                serverTimestamps: options === null || options === void 0 ? void 0 : options.serverTimestamps,
                                            });
                                            if (options === null || options === void 0 ? void 0 : options.idField) {
                                                data = __assign(__assign({}, data), (_a = {}, _a[options.idField] = doc.id, _a));
                                            }
                                            return data;
                                        });
                                        if (!resolved) {
                                            resolved = true;
                                            return resolve(docs);
                                        }
                                        else {
                                            client.setQueryData(key, docs);
                                        }
                                    }, reject);
                                })];
                    }
                });
            });
        } }));
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
function useFirestoreCollectionMutation(ref, useMutationOptions) {
    return useMutation(function (data) {
        return addDoc(ref, data);
    }, useMutationOptions);
}
function useFirestoreDocumentMutation(ref, options, useMutationOptions) {
    return useMutation(function (data) {
        if (options) {
            return setDoc(ref, data, options);
        }
        return setDoc(ref, data);
    }, useMutationOptions);
}
function useFirestoreDocumentDeletion(ref, useMutationOptions) {
    return useMutation(function () { return deleteDoc(ref); }, useMutationOptions);
}
function useFirestoreTransaction(firestore, updateFunction, useMutationOptions) {
    return useMutation(function () {
        return runTransaction(firestore, updateFunction);
    }, useMutationOptions);
}
function useFirestoreWriteBatch(batch, useMutationOptions) {
    return useMutation(function () {
        return batch.commit();
    }, useMutationOptions);
}

var namedQueryCache = {};
function namedQuery(firestore, name) {
    var key = firestore.app.name + ":" + name;
    if (namedQueryCache[key]) {
        return namedQueryCache[key];
    }
    return function () {
        return namedQuery$1(firestore, name).then(function (query) {
            if (query) {
                namedQueryCache[key] = query;
                return query;
            }
            return null;
        });
    };
}

/*
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
function getSnapshot(ref, source) {
    return __awaiter(this, void 0, void 0, function () {
        var snapshot;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(source === "cache")) return [3 /*break*/, 2];
                    return [4 /*yield*/, getDocFromCache(ref)];
                case 1:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 2:
                    if (!(source === "server")) return [3 /*break*/, 4];
                    return [4 /*yield*/, getDocFromServer(ref)];
                case 3:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 4: return [4 /*yield*/, getDoc(ref)];
                case 5:
                    snapshot = _a.sent();
                    _a.label = 6;
                case 6: return [2 /*return*/, snapshot];
            }
        });
    });
}
function getQuerySnapshot(query, source) {
    return __awaiter(this, void 0, void 0, function () {
        var snapshot;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!(source === "cache")) return [3 /*break*/, 2];
                    return [4 /*yield*/, getDocsFromCache(query)];
                case 1:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 2:
                    if (!(source === "server")) return [3 /*break*/, 4];
                    return [4 /*yield*/, getDocsFromServer(query)];
                case 3:
                    snapshot = _a.sent();
                    return [3 /*break*/, 6];
                case 4: return [4 /*yield*/, getDocs(query)];
                case 5:
                    snapshot = _a.sent();
                    _a.label = 6;
                case 6: return [2 /*return*/, snapshot];
            }
        });
    });
}
function isNamedQuery(query) {
    return typeof query === "function";
}
function resolveQuery(query) {
    return __awaiter(this, void 0, void 0, function () {
        var resolved;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!isNamedQuery(query)) return [3 /*break*/, 3];
                    if (!(typeof query === "function")) return [3 /*break*/, 2];
                    return [4 /*yield*/, query()];
                case 1:
                    resolved = _a.sent();
                    return [2 /*return*/, resolved];
                case 2: return [2 /*return*/, query];
                case 3: return [2 /*return*/, query];
            }
        });
    });
}

export { getQuerySnapshot, getSnapshot, namedQuery, resolveQuery, useFirestoreCollectionMutation, useFirestoreDocument, useFirestoreDocumentData, useFirestoreDocumentDeletion, useFirestoreDocumentMutation, useFirestoreInfiniteQuery, useFirestoreInfiniteQueryData, useFirestoreQuery, useFirestoreQueryData, useFirestoreTransaction, useFirestoreWriteBatch };
